#ifndef LH_MAIN_H
#define LH_MAIN_H

#define SIZE(ARR) sizeof(ARR) / sizeof(ARR[0])



/*
 * Structure for interrupt id, handler and callback reference
 */
typedef struct {
	u8 id;
	XInterruptHandler handler;
	void *pvCallbackRef;
} ivt_t;

#define AUDIO_SAMPLING_RATE	  96000 //48000 samples per second but 2 samples per "frame"
#define NR_AUDIO_SAMPLES	 (NR_SEC_TO_REC_PLAY*AUDIO_SAMPLING_RATE) //stereo, 2 samples for every frame

// Number of seconds to record/playback
#define NR_SEC_TO_REC_PLAY		5


#define SINE_LEN	8192
#define PI 			3.14159265

#define C	0
#define Db	1
#define D	2
#define Eb	3
#define E	4
#define F	5
#define Gb	6
#define G	7
#define Ab	8
#define A	9
#define Bb	10
#define B	11

#define NUM_NOTES	12
#define NUM_STRINGS	12//15

#define BUFFER_SIZE  AUDIO_SAMPLING_RATE/16 //size of output sound buffer

#define BASE_MULT  1
double base_freqs[] = {65.40639 *BASE_MULT,  //C5
					   69.29566*BASE_MULT,  //Db
					   73.41619*BASE_MULT,  //D
					   77.78175*BASE_MULT,  //Eb
					   82.40689*BASE_MULT,  //E
					   87.30706*BASE_MULT,  //F
				       92.49861*BASE_MULT,  //Gb
					   97.99885*BASE_MULT,  //G
					   103.82617*BASE_MULT, //Ab
					   110.00000*BASE_MULT, //A
					   116.54094*BASE_MULT, //Bb
					   123.47083*BASE_MULT, //B
};

#define NUM_HARMONICS	7

#define MAX_MENU_SIZE	8

typedef struct menu
{
	int selected;
	char * display[MAX_MENU_SIZE];
	int size;
	struct menu * child[MAX_MENU_SIZE];
	struct menu * prev;
	void (*fun)(void);
}menu_t;

typedef struct
{
	int is_active;
	uint8_t note; //the base note
	uint8_t scale[NUM_HARMONICS]; //scale of each harmonic frequency
	uint32_t time; //for indexing and when to stop playing
	uint32_t duration; //stop playing when time = duration
} sound_t;

#define MAX_DECAY	AUDIO_SAMPLING_RATE/2 //how many points in decay table
#define DECAY_TIME_SCALE 64 //every decay value holds for 32 frames



typedef struct
{
	int32_t value[NUM_STRINGS][2000];
	uint16_t length[NUM_STRINGS];
	uint8_t octave[NUM_STRINGS];
	uint16_t decay[NUM_HARMONICS][MAX_DECAY];
	uint16_t decay_const_index[NUM_HARMONICS]; //index into a list of possible decay rates
}table_t;

#define MAX_INTERVALS	NUM_NOTES
typedef struct
{
	int length;
	int intervals[MAX_INTERVALS];
	char * name;
}scale_t;

#endif
